using System;
using System.Collections;
using PanelApplet;
using Gtk;
using Beagle;

//using Best;

namespace BeagleApplet
{
	public class BeagleQueryApplet : PanelApplet
	{

		protected BeagleQueryApplet (IntPtr raw) : base (raw) {}

		static void Main (string[] argv)
		{
			Gnome.Program p = new Gnome.Program ("BeagleQueryApplet", "0.1", Gnome.Modules.UI, argv);
			AppletFactory.Register (typeof (BeagleQueryApplet));
		}

		private HBox hbox;
		private Entry searchText;
		private Button submit;
		private String currentSearch;
		private bool resultsWindowHidden;

		public override void Creation ()
		{
			hbox = new HBox ();
			searchText = new Entry ();
			submit = new Button ("Search");
			submit.Clicked += new EventHandler (run_query);
			
			resultsWindowHidden = false;
			currentSearch = "";
			
			hbox.PackStart (searchText);
			hbox.PackStart (submit);
			this.Add (hbox);
			this.ShowAll ();
		}
				
		private Window resultsWin;
		private ScrolledWindow scrollWin;

		private QueryDriver driver;
		private QueryResult result;
		
		private TileCanvas canvas;
		private AppletRootTile root;		
		
		private void OnResults (QueryResult src, QueryResult.GotHitsArgs args)
		{
			if (src != result)
				return;
			
			foreach (Hit hit in args.Hits)
				root.Add (hit);
		}

		private void OnFinished (QueryResult src)
		{
			if (src != result)
				return;
			Console.WriteLine ("Finished!");
			root.Close ();
		}

		private void OnCancelled (QueryResult src)
		{
			if (src != result)
				return;
			Console.WriteLine ("Cancelled!");
			root.Close ();
		}

		private void OnPostRender (object obj, EventArgs args)
		{
			System.Console.WriteLine ("OnPostRender");
			if (canvas == null)
				return;
			System.Console.WriteLine ("canvas.Height = {0}", (int)canvas.Height);
			resultsWin.Resize (resultsWin.DefaultWidth, (int)canvas.Height);
		}
		
		private void run_query (object obj, EventArgs args)
		{
			// Run beagle search stuff
			if (searchText.Text != currentSearch)
			{
				currentSearch = searchText.Text; 
				System.Console.WriteLine ("Initiating search for {0}", searchText.Text);
				Query query = new Query (searchText.Text);
				query.AddDomain (QueryDomain.Neighorhood);
				query.AddDomain (QueryDomain.Global);
	
				if (result != null)
					result.Cancel ();
	
				if (driver == null)
					driver = new QueryDriver ();
					
				System.Console.WriteLine ("Generating result");
				result = driver.Query (query);
				result.GotHitsEvent += OnResults;
				result.FinishedEvent += OnFinished;
				result.CancelledEvent += OnCancelled;
				
				// Start up the GUI
			
				if (resultsWin == null)
				{
					resultsWin = new Window ("Search results");
					scrollWin = new ScrolledWindow ();
					resultsWin.Add (scrollWin);
					resultsWin.TypeHint = Gdk.WindowTypeHint.Dock;
		
					canvas = new TileCanvas ();
					canvas.PostRenderEvent += new EventHandler (OnPostRender);
					root = new AppletRootTile ();
					canvas.Root = root;
				
					scrollWin.Add (canvas);

					resultsWin.DefaultWidth = searchText.Allocation.Width + submit.Allocation.Width;
					resultsWin.DefaultHeight = 0;
					
					position_results_window (resultsWin, searchText);

				}
				else
				{
					root = new AppletRootTile ();
					canvas.Root = root;
				}		

				System.Console.WriteLine ("Opening root");
				root.Open ();
				
				result.Start ();				
				resultsWin.ShowAll ();
				resultsWindowHidden = false;
			}
			else
			{
				if (!resultsWindowHidden)
					resultsWin.HideAll ();
				else
					resultsWin.ShowAll ();
				resultsWindowHidden = !resultsWindowHidden;
			}
			
		}

		private void
		position_results_window (Window window,
                	        	 Entry entry)
		{
			/* Code copied/ported from the clock applet in gnome-panel */
			Gtk.Requisition req;
			Gdk.Screen      screen;
			Gdk.Rectangle   monitor;
			int             entry_w, entry_h;
			int             x, y;
			int             w, h;
			int             i, n;
			bool            found_monitor = false;

			/* Get root origin of the entry, and position near that. */
			entry.GdkWindow.GetOrigin (out x, out y);
		
			//window.GetSize (w, h);
			req = window.SizeRequest ();
			w = req.Width;
			h = req.Height;

			entry_w = entry.Allocation.Width;
			entry_h = entry.Allocation.Height;

			screen = window.Screen;

			n = screen.NMonitors;
			for (i = 0; i < n; i++) {
				monitor = screen.GetMonitorGeometry (i);
				if (x >= monitor.X && x <= monitor.X + monitor.Width &&
				    y >= monitor.Y && y <= monitor.Y + monitor.Height) {
					found_monitor = true;
					break;
				}
			}


			if ( ! found_monitor) {
				/* eek, we should be on one of those xinerama
				   monitors */
				monitor.X = 0;
				monitor.Y = 0;
				monitor.Width = screen.Width;
				monitor.Height = screen.Height;
			}
		
			/* Based on panel orientation, position the popup.
			 * Ignore window gravity since the window is undecorated.
			 * The orientations are all named backward from what
			 * I expected.
			 */
			// Gtk# doesn't do these enums yet, so for now assume that its facing downwards (ie a top panel)
			/*switch (GetOrient) {
			case PANEL_APPLET_ORIENT_RIGHT:
				x += entry_w;
				if ((y + h) > monitor.Y + monitor.Height)
					y -= (y + h) - (monitor.Y + monitor.Height);
				break;
			case PANEL_APPLET_ORIENT_LEFT:
				x -= w;
				if ((y + h) > monitor.Y + monitor.Height)
					y -= (y + h) - (monitor.Y + monitor.Height);
				break;
			case PANEL_APPLET_ORIENT_DOWN: */
				y += entry_h;
				if ((x + w) > monitor.X + monitor.Width)
					x -= (x + w) - (monitor.X + monitor.Width);
			/*	break;
			case PANEL_APPLET_ORIENT_UP:
				y -= h;
				if ((x + w) > monitor.X + monitor.Width)
					x -= (x + w) - (monitor.X + monitor.Width);
				break;
			}*/
				
			window.Move (x, y);
		}

		
		public override string IID {
			get { return "OAFIID:BeagleQueryApplet"; }
		}

		public override string FactoryIID {
			get { return "OAFIID:BeagleQueryApplet_Factory"; }
		}
	}
}
